/*
 * Tapiocaui app
 * Copyright (C) 2005 INdT
 * @author Renato Araujo Oliveira Filho <renato.filho@indt.org.br>
 * @author Andre Moreira Magalhaes <andre.magalhaes@indt.org.br>
 * @author Abner Jose de Faria Silva <abner.silva@indt.org.br>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA02111-1307USA
 */

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include "tapiocaui_shell_communication.h"
#include "tapiocaui_shell_session.h"
#include "tapiocaui_shell_media.h"

void
tapiocaui_shell_communication_composing_cb (TapiocaUICommunication *comm,
                                            gpointer user_data)
{
    TapiocaUIShell *self = TAPIOCAUI_SHELL (user_data);
    TapiocaUIShellActiveComm *active_comm;
    const gchar *body = "", *subject = "";

    active_comm = g_hash_table_lookup (self->priv->active_comm_list, tapiocaui_communication_get_buddy (comm)->id);

    if (!active_comm || !active_comm->current_text_session) {
        return;
    }

    g_mutex_lock (active_comm->mutex);

    active_comm->idle_time = 0;

    if (active_comm->last_notification_state != TAPIOCA_TEXT_SESSION_NOTIFICATION_STATE_COMPOSING) {
        active_comm->last_notification_state = TAPIOCA_TEXT_SESSION_NOTIFICATION_STATE_COMPOSING;
        tapioca_text_session_send (active_comm->current_text_session,
                TAPIOCA_TEXT_SESSION_MESSAGE_TYPE_NONE,
                subject, body,
                TAPIOCA_TEXT_SESSION_NOTIFICATION_STATE_COMPOSING);

        if (active_comm->idle_source == -1)
            active_comm->idle_source = g_timeout_add (1000, (GSourceFunc) inc_idle_time, active_comm);
    }

    g_mutex_unlock (active_comm->mutex);
}

gboolean
inc_idle_time (TapiocaUIShellActiveComm *comm)
{
    TapiocaTextSessionNotificationState state = TAPIOCA_TEXT_SESSION_NOTIFICATION_STATE_NONE;
    const gchar *body = "", *subject = "";
    gboolean ret = TRUE;

    g_mutex_lock (comm->mutex);

    ++comm->idle_time;

    if (comm->idle_time == 5) {
        state = TAPIOCA_TEXT_SESSION_NOTIFICATION_STATE_PAUSED;
    }
    else if (comm->idle_time == 120) {
        state = TAPIOCA_TEXT_SESSION_NOTIFICATION_STATE_INACTIVE;
        comm->idle_source = -1;
        ret = FALSE;
    }

    if (state != TAPIOCA_TEXT_SESSION_NOTIFICATION_STATE_NONE) {
        comm->last_notification_state = state;
        tapioca_text_session_send (comm->current_text_session,
                TAPIOCA_TEXT_SESSION_MESSAGE_TYPE_NONE,
                subject, body, state);
    }

    g_mutex_unlock (comm->mutex);

    return ret;
}

void
tapiocaui_shell_communication_send_message_cb (TapiocaUICommunication *comm,
                                               const gchar *msg,
                                               gpointer user_data)
{
    TapiocaUIShell *self = TAPIOCAUI_SHELL (user_data);
    TapiocaUIShellActiveComm *active_comm;
    TapiocaTextSession *text_session;
    gchar **members = NULL;
    const gchar *subject = "";

    active_comm = g_hash_table_lookup (self->priv->active_comm_list, tapiocaui_communication_get_buddy (comm)->id);
    if (!active_comm) {
        return;
    }

    if (!(active_comm->text_sessions)) {
        tapiocaui_debug ("creating a new text session and add members");
        text_session = (TapiocaTextSession *) tapioca_connection_create_session (self->priv->tc,
                TAPIOCA_SESSION_TYPE_TEXT);

        active_comm->text_sessions = g_list_append (active_comm->text_sessions, text_session);

        /* Connect session signals */
        g_signal_connect (G_OBJECT (text_session),
                          "member-added",
                          G_CALLBACK (tapiocaui_shell_session_member_added_cb),
                          self);
        g_signal_connect (G_OBJECT (text_session),
                          "member-removed",
                          G_CALLBACK (tapiocaui_shell_session_member_removed_cb),
                          self);
        g_signal_connect (G_OBJECT (text_session),
                          "add-member-error",
                          G_CALLBACK (tapiocaui_shell_session_add_member_er_cb),
                          self);
        g_signal_connect (G_OBJECT (text_session),
                          "state-changed",
                          G_CALLBACK (tapiocaui_shell_session_state_changed_cb),
                          self);
        g_signal_connect (G_OBJECT (text_session),
                          "error",
                          G_CALLBACK (tapiocaui_shell_session_error_signal_cb),
                          self);

        /* Connect text session signals */
        g_signal_connect (G_OBJECT (text_session),
                          "incoming",
                          G_CALLBACK (tapiocaui_shell_text_session_incoming_cb),
                          self);

        g_signal_connect (G_OBJECT (text_session),
                          "delivery-error",
                          G_CALLBACK (tapiocaui_shell_text_session_derror_cb),
                          self);

        tapioca_text_session_disable_message_queue (text_session);

        members = g_new0 (gchar *, 2);
        members[0] = tapiocaui_communication_get_buddy (comm)->id;
        tapioca_session_add_members (TAPIOCA_SESSION (text_session), (const gchar **) members);

        if (members) {
            g_free (members);
            members = NULL;
        }

        tapioca_session_initiate (TAPIOCA_SESSION (text_session));

        active_comm->current_text_session = text_session;
    }

    g_mutex_lock (active_comm->mutex);
    tapioca_text_session_send (active_comm->current_text_session,
            TAPIOCA_TEXT_SESSION_MESSAGE_TYPE_PLAIN_TEXT,
            subject,
            msg,
            TAPIOCA_TEXT_SESSION_NOTIFICATION_STATE_ACTIVE);

    active_comm->last_notification_state = TAPIOCA_TEXT_SESSION_NOTIFICATION_STATE_ACTIVE;
    g_mutex_unlock (active_comm->mutex);
}

//accept incomming call
void
tapiocaui_shell_communication_call_answer_cb (TapiocaUICommunication *comm,
                                              const gchar *user,
                                              gpointer user_data)
{
    TapiocaUIShellActiveComm *active_comm;
    TapiocaUIShell *self = TAPIOCAUI_SHELL (user_data);
    gchar *local_cap = NULL;

    active_comm = tapiocaui_shell_get_communication (self, user);

    local_cap = create_sdp_from_codecs (farsight_stream_get_local_codecs (active_comm->stream));

    tapiocaui_debug ("set local capabilities %s", local_cap);

    if (!(tapioca_audio_session_set_local_capabilities (
                    active_comm->audio_session,
                    (const gchar *) local_cap))) {
        tapiocaui_debug ("error on set local cap");
        g_object_unref (active_comm->audio_session);
        active_comm->audio_session = NULL;
        g_free (local_cap);
        return;
    }

    g_free (local_cap);

    if (!tapioca_audio_session_accept (active_comm->audio_session)) {
        tapiocaui_communication_set_status (active_comm->comm_win,
                                            TAPIOCAUI_CALL_IDLE);
    }
}

//reject incomming call
void
tapiocaui_shell_communication_call_reject_cb (TapiocaUICommunication *comm,
                                              const gchar *user,
                                              gpointer user_data)
{
    TapiocaUIShellActiveComm *active_comm;
    TapiocaUIShell *self = TAPIOCAUI_SHELL (user_data);

    active_comm = tapiocaui_shell_get_communication (self, user);

    tapiocaui_debug ("Rejecting a call ...");

    if (!(tapioca_audio_session_reject (active_comm->audio_session)))
        tapiocaui_debug ("Failed on reject a call");

    tapiocaui_shell_close_call (self, active_comm);
}

//disconnect incomming call
void
tapiocaui_shell_communication_call_disconnect_cb (TapiocaUICommunication *comm,
                                                  const gchar *user,
                                                  gpointer user_data)
{
    TapiocaUIShellActiveComm *active_comm;
    TapiocaUIShell *self = TAPIOCAUI_SHELL (user_data);

    active_comm = tapiocaui_shell_get_communication (self, user);
    tapiocaui_shell_close_call (self, active_comm);
}

//make a call
void
tapiocaui_shell_communication_call_make_cb (TapiocaUICommunication *comm,
                                            const gchar *user,
                                            gpointer user_data)
{
    TapiocaUIShellActiveComm *active_comm;
    TapiocaUIShell *self = TAPIOCAUI_SHELL (user_data);

    tapiocaui_debug ("trying to make a call");
    active_comm = tapiocaui_shell_get_communication (self, user);
    tapiocaui_shell_call_buddy (self, active_comm);
}

void
tapiocaui_shell_communication_call_hold_cb (TapiocaUICommunication *comm,
                                            const gchar *user,
                                            gpointer user_data)
{
    tapiocaui_debug ("hold not implemented");
    // tapiocaui_communication_set_status (comm, TAPIOCAUI_CALL_HOLD);
}

void
tapiocaui_shell_communication_call_unhold_cb (TapiocaUICommunication *comm,
                                              const gchar *user,
                                              gpointer user_data)
{
    tapiocaui_debug ("unhold not implemented");
    // tapiocaui_communication_set_status (comm, TAPIOCAUI_CALL_ANSWERED);
}

// when communication window is closed
void
tapiocaui_shell_communication_quit_cb (TapiocaUICommunication *comm,
                                       gpointer user_data)
{
    TapiocaUIShellActiveComm *active_comm;
    const TapiocaUIBuddyData *buddy;
    TapiocaUIShell *self = TAPIOCAUI_SHELL (user_data);
    GList *text_sessions;
    TapiocaTextSession *text_session;
    const gchar *subject = "", *body = "";

    tapiocaui_debug ("communication window closed");
    buddy = tapiocaui_communication_get_buddy (comm);

    active_comm = tapiocaui_shell_get_communication (self, buddy->id);

    g_mutex_lock (active_comm->mutex);

    if (active_comm->idle_source != -1)
        g_source_remove (active_comm->idle_source);

    tapiocaui_shell_close_call (self, active_comm);

    for (text_sessions = active_comm->text_sessions; text_sessions; text_sessions = g_list_next (text_sessions)) {
        text_session = (TapiocaTextSession *) text_sessions->data;
        tapioca_text_session_send (text_session,
                TAPIOCA_TEXT_SESSION_MESSAGE_TYPE_NONE,
                subject, body,
                TAPIOCA_TEXT_SESSION_NOTIFICATION_STATE_GONE);
        g_object_unref (text_session);
    }
    g_list_free (active_comm->text_sessions);
    active_comm->text_sessions = NULL;

    g_hash_table_remove (self->priv->active_comm_list, buddy->id);

    self->priv->active_comm = NULL;

    g_mutex_unlock (active_comm->mutex);
    g_mutex_free (active_comm->mutex);

    g_free (active_comm);
}

void
tapiocaui_shell_communication_volume_change_cb (TapiocaUICommunication *comm,
                                                guint volume,
                                                gpointer user_data)
{
    /*
    TapiocaUIShell *self = TAPIOCAUI_SHELL (user_data);
    if (volume > 100)
        volume = 100;
    tapiocaui_debug ("volume change to %d", volume);
    g_object_set (G_OBJECT (self->priv->active_comm->sinkvolume), "volume", (double) volume / 25, NULL);
    */

    tapiocaui_debug ("Not implemented");
}

void
tapiocaui_shell_communication_input_mute_cb (TapiocaUICommunication *comm,
                                             gpointer user_data)
{
    TapiocaUIShell *self = TAPIOCAUI_SHELL (user_data);
    tapiocaui_debug ("input mute");
    g_object_set (G_OBJECT (self->priv->active_comm->srcvolume), "mute", TRUE, NULL);
}

void
tapiocaui_shell_communication_input_unmute_cb (TapiocaUICommunication *comm,
                                               gpointer user_data)
{
    TapiocaUIShell *self = TAPIOCAUI_SHELL (user_data);
    tapiocaui_debug ("input unmute");
    g_object_set (G_OBJECT (self->priv->active_comm->srcvolume), "mute", FALSE, NULL);
}
