/*
 * Tapiocaui app
 * Copyright (C) 2005 INdT
 * @author Marcio Macedo  <marcio.macedo@indt.org.br>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA02111-1307USA
 */

#include <gtk/gtk.h>
#include <glade/glade-xml.h>

#include "tapiocaui_systray.h"
#include "eggtrayicon.h"
#include "tapiocaui_utils.h"


static GdkPixbuf *systray_load_icon             (const gchar *filename);
static GdkPixbuf *systray_pixbuf_new_from_file  (const gchar *filename);

static void systray_destroyed           (GtkWidget *widget, gpointer data);
static void systray_quit_cb             (GtkWidget *widget, gpointer data);
static void systray_logoff_cb           (GtkWidget *widget, gpointer data);
static void systray_clicked             (GtkWidget *widget, GdkEventButton *event, void *data);
static void systray_show_hidden_windows        (gpointer data, gpointer ignored);
static void systray_show_main_window    (GtkWidget *window);
static gboolean systray_generate_menu   (GdkEventButton *event, TapiocaUIMainWin *main_win);
static gboolean flash_icon_calling      (gpointer data);
void stop_icon_calling                  (void);
        

/* global control variables and icons */
static GdkPixbuf *icon_offline, *icon_online, *icon_calling;
static GtkWidget *image_icon;
gint flash_id;
GList *lst_hidden_windows;

EggTrayIcon *tray_icon;
GtkTooltips *tray_tooltip;

void 
tapiocaui_systray_init(TapiocaUIMainWin *main_window) 
{
	GtkWidget *eventbox;
	
	/* tray icon */
	image_icon = gtk_image_new();
	tray_icon = egg_tray_icon_new("tapioca");
	eventbox = gtk_event_box_new();
	tray_tooltip = gtk_tooltips_new();

	/* icon list */
	icon_offline = systray_load_icon(tapiocaui_file("systray_offline.png"));
	icon_online = systray_load_icon(tapiocaui_file("systray_online.png"));
	icon_calling = systray_load_icon(tapiocaui_file("systray_calling.png"));
	
    flash_id = 0;
    lst_hidden_windows = NULL;
    
	tapiocaui_systray_set_icon_offline();

	gtk_container_add(GTK_CONTAINER(eventbox), GTK_WIDGET(image_icon));
	gtk_container_add(GTK_CONTAINER(tray_icon), eventbox);

	gtk_widget_show_all(GTK_WIDGET(tray_icon));	
	
	g_signal_connect(G_OBJECT(tray_icon), 
			 "destroy", 
			 G_CALLBACK(systray_destroyed), 
			 NULL);
	g_signal_connect(G_OBJECT(eventbox), 
			 "button-press-event", 
			 G_CALLBACK(systray_clicked), 
			 main_window);						

	egg_tray_icon_send_message(tray_icon,1000,"tapioca",7);
}

static
GdkPixbuf *systray_load_icon (const gchar *filename)
{
	gint w = 0;
  	gint h = 0;

  	GdkPixbuf *pb = NULL;
  	GdkPixbuf *pb_scaled = NULL;
	
	
	if((pb = systray_pixbuf_new_from_file(filename)) == NULL) {
		g_warning("systray_load_icon: pixbuf was NULL\n");
		return NULL;
	}

 	/* get size */
	gtk_icon_size_lookup(GTK_ICON_SIZE_LARGE_TOOLBAR, &w, &h);

	/* scale the image */
	pb_scaled = gdk_pixbuf_scale_simple(pb, w, h, GDK_INTERP_BILINEAR);

	/* clean up */
 	g_object_unref(G_OBJECT(pb));
	
    return pb_scaled;
}

static GdkPixbuf *
systray_pixbuf_new_from_file(const gchar *filename)
{
	GdkPixbuf *pb = NULL;
	GError *error = NULL;

	if (filename == NULL || g_utf8_strlen(filename, -1) < 1) {
		g_warning("%s: filename was NULL", __FUNCTION__);
		return NULL;
	}
  
	pb = gdk_pixbuf_new_from_file(filename, &error);

	if(pb == NULL) {
		g_warning("%s: error loading file:'%s'", __FUNCTION__, filename);
	
		/* look at error */
		if(error != NULL) {
			g_warning("%s: error domain:'%s', code:%d, message:'%s'", __FUNCTION__,
			g_quark_to_string(error->domain), error->code, error->message);
			g_error_free(error);
		}
	
		return NULL;
	}

	/* NOTE: this has a refcount of 1 it needs to be unref'd */
	return pb;
}

static gboolean 
systray_generate_menu(GdkEventButton *event, TapiocaUIMainWin *main_win)
{
	GtkWidget *systray_menu;
	GtkWidget *status_menu;
	GtkWidget *item = NULL;
	
	if(event == NULL) {
		g_warning("systray_generate_menu: gdk event was NULL");
		return FALSE;
	}
	
	systray_menu = gtk_menu_new();

	
    /* TODO: implement a signal hancdler for this item */
    /*
	item = gtk_menu_item_new_with_label("Chat/Call...");
	gtk_menu_shell_append(GTK_MENU_SHELL(systray_menu), item);
    */
	status_menu = gtk_menu_new();

    /* TODO: implement status control on context menu */
    /*
	item = gtk_menu_item_new_with_label("Status");
	gtk_menu_item_set_submenu(GTK_MENU_ITEM(item),status_menu);
	gtk_menu_shell_append(GTK_MENU_SHELL(systray_menu),item);
	*/

	item = gtk_menu_item_new_with_label("Log Out");
	gtk_menu_shell_append(GTK_MENU_SHELL(systray_menu), item);

	gtk_signal_connect (GTK_OBJECT (item), "activate", 
			    GTK_SIGNAL_FUNC(systray_logoff_cb), 
			    main_win);

	item = gtk_separator_menu_item_new();
	gtk_menu_shell_append(GTK_MENU_SHELL(systray_menu), item);
	item = gtk_image_menu_item_new_from_stock(GTK_STOCK_QUIT, NULL);
	gtk_menu_shell_append(GTK_MENU_SHELL(systray_menu), item);

	gtk_signal_connect (GTK_OBJECT (item), "activate", 
			    GTK_SIGNAL_FUNC(systray_quit_cb), 
			    NULL);

	/* show */
	gtk_widget_show_all(GTK_WIDGET(systray_menu));
	gtk_menu_popup(GTK_MENU(systray_menu), NULL, NULL, NULL, NULL, event->button, event->time);

	return TRUE;

}

static void 
systray_quit_cb(GtkWidget *widget, gpointer data)
{
    gtk_main_quit();
}

static void
systray_logoff_cb(GtkWidget *widget, gpointer data)
{
    g_signal_emit_by_name(TAPIOCAUI_MAIN_WIN(data), "logout", NULL);
}

static void 
systray_show_main_window(GtkWidget *window) 
{
    static gint position_x; 
    static gint position_y;

     position_x = position_y = 0;

    if(lst_hidden_windows)
    {
       g_list_foreach(lst_hidden_windows, systray_show_hidden_windows, NULL);
       g_list_free(lst_hidden_windows);
       lst_hidden_windows = NULL;
       stop_icon_calling();
       return;
    }

	if((gdk_window_get_state(GTK_WIDGET(window)->window) & 
				 GDK_WINDOW_STATE_ICONIFIED) || 
			         !GTK_WIDGET_VISIBLE(window))
    { 
        gtk_window_move(GTK_WINDOW(window), position_x, position_y);
		gtk_window_present(GTK_WINDOW(window));
    }
	else 
    {
        gtk_window_get_position(GTK_WINDOW(window), &position_x, &position_y);
		gtk_widget_hide(GTK_WIDGET(window));
    }
}

static void 
systray_destroyed(GtkWidget *widget, gpointer data)
{
	systray_show_main_window(GTK_WIDGET(data));
}

static void 
systray_clicked(GtkWidget *widget, GdkEventButton *event, void *data)
{
	
	if (event->button == 3) {
		systray_generate_menu(event, TAPIOCAUI_MAIN_WIN(data));
	} else {
		systray_show_main_window(GTK_WIDGET(data));
	}
}

void 
tapiocaui_systray_set_icon_online()
{
	/* gtk_image_set_from_pixbuf(GTK_IMAGE(image_icon), icon_online); */
    stop_icon_calling();
}

void
tapiocaui_systray_set_icon_calling()
{
	gtk_image_set_from_pixbuf(GTK_IMAGE(image_icon), icon_calling);
    if(!flash_id)
    	flash_id = g_timeout_add (500, flash_icon_calling, NULL); 
}

static gboolean
flash_icon_calling (gpointer data)
{
    static int i = 0;
    
    i++;
    if(i == 2){
    	gtk_image_set_from_pixbuf(GTK_IMAGE(image_icon), icon_online);
        i = 0;
    }
    else
    	gtk_image_set_from_pixbuf(GTK_IMAGE(image_icon), icon_calling);
    
	return TRUE;
}

void stop_icon_calling()
{
    if(flash_id)
    {
        g_source_remove(flash_id);
        flash_id = 0;
    }
    gtk_image_set_from_pixbuf(GTK_IMAGE(image_icon), icon_online);
}

void
tapiocaui_systray_set_icon_offline()
{
	gtk_image_set_from_pixbuf(GTK_IMAGE(image_icon), icon_offline);
}
void 
tapiocaui_systray_add_hidden_window(GtkWidget *window)
{
    lst_hidden_windows = g_list_append(lst_hidden_windows, window);
}

static void 
systray_show_hidden_windows(gpointer data, gpointer ignored)
{
    gtk_window_present(GTK_WINDOW(data));
}
